/***************************************************************************
 *            netgetfile.c
 *
 *  Fri Oct 24 11:27:36 2003
 *  Copyright  2003  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/

#include "../include/remsync.h"

void sig_alarm(int sig) 
{
	/* Do Nothing */
}


int
netgetfile (char *src_file, char *dest_file, int port, char *servername,
	    int filemode, int uid, int gid, int mtime, int atime)
{
	int client_socket;
	char command[BIG_BUF], message[BIG_BUF];
	int block_len = 8192;
	char asize[BIG_BUF], acsum[BIG_BUF];
	char dummy1[BIG_BUF], dummy2[BIG_BUF], filename[BIG_BUF],
		buffer[block_len];
	int csum, rnd, local_csum;
	unsigned long long int size, sizeread, total_read;
	FILE *client_file;
	time_t trans_start, trans_finish;
	int trans_time_taken;
	int local_file, set_alarm = 0;
	struct utimbuf times;
	struct sigaction act;
	

	client_socket = init_client (port, servername);
	client_file = fdopen (client_socket, "r+");

	sprintf (command, "send|%s\r\n", src_file);
	send (client_socket, command, strlen (command), 0);

	fgets (message, BIG_BUF, client_file);
	strip_string (message);
	split_string (message, dummy1, dummy2);
	split_string (dummy2, asize, dummy1);
	split_string (dummy1, asize, acsum);
	size = atoll (asize);
	csum = atoi (acsum);
	times.actime = (time_t)atime;
	times.modtime = (time_t)mtime;
	sprintf (command, "OK\r\n");
	send (client_socket, command, strlen (command), 0);

	rnd = rand ();
	sprintf (filename, "%sremosync.%d",conf.dest, rnd);
	local_file =
		open (filename,
		      O_CREAT | O_TRUNC | O_WRONLY | O_BINARY, 0777);
	if (local_file == -1) {
		syslog(LOG_ERR,"Failed to open file %s (%m)",filename);
		return(1);
	}
	if (conf.debug) printf ("Local File %s\nRemote File %s\nSize %Ld\n",filename, src_file, size);
	sizeread = 0;
	total_read = 0;
	if (size != 0)
	{
		/* set up alarm */
		bzero(&act, sizeof(act));
		act.sa_handler = sig_alarm;
		act.sa_flags = SA_ONESHOT;
		
		/* Set up handler */
		if (sigaction(SIGALRM, &act, 0) != 0)
			syslog(LOG_ERR,"Unable to set up timeout (netgetfile)!");
		else 
			set_alarm = 1;
		trans_start = time(NULL);
		bzero (buffer, block_len);
		while ((total_read < size))
		{
			if (set_alarm == 1) 
				alarm(500); /* 500 seconds timeout */
			sizeread = read (client_socket, buffer, block_len);
			if (sizeread < 0) {
				syslog(LOG_ERR, "Timeout (netgetfile)");
				if (set_alarm == 1) 
					alarm(0); /* Reset alarm */
				break;
			}
			if (set_alarm == 1) 
				alarm(0); /* Reset alarm */
			
			total_read += sizeread;
			write (local_file, buffer, sizeread);
		}
		trans_finish = time(NULL);
		trans_time_taken = trans_finish - trans_start;
		if (trans_time_taken == 0) {
			stats.transfer_time += 0.5;
		} else {
			stats.transfer_time += trans_time_taken;
		}
	} else {
		//Handle the case of zero byte files
		// seems to cause problems with the handshaking
		sleep (1);
	}
	stats.total_bytes_copied += size;
	local_csum = cksum (filename);
	if (local_csum != csum)
	{
		if (conf.debug) printf ("csum Local = %d, Remote = %d\n", local_csum, csum);
		syslog(LOG_ERR,"Checksums wrong! %s", filename);
		unlink (filename);
		return (1);
	}
	else
	{
		if (conf.debug) syslog(LOG_DEBUG, "Renaming file %s to %s\n", filename, dest_file);
		//rename (filename, dest_file);
		copy_file (filename, dest_file);
		chmod (dest_file, filemode);
		chown (dest_file, uid, gid);
		if (conf.timestamp == 1) {
			utime (dest_file, &times);
		}		
		unlink (filename);

	}
	if (conf.debug) printf("Got local file, closing and quitting\n");
	close (local_file);
	client_quit (client_socket);
	fclose (client_file);
	close (client_socket);
	return (0);
}
